package sacooliveros.whatsappweb.api;

import java.util.Map;
import java.util.logging.Logger;
import org.apache.catalina.connector.Response;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import it.auties.whatsapp.api.Whatsapp;
import sacooliveros.whatsappweb.bean.WhatsappBean;
import sacooliveros.whatsappweb.service.WhatsappService;
import sacooliveros.whatsappweb.utils.WhatsappUtils;

@RestController
@CrossOrigin("*")
@RequestMapping("api/v1")
public class WhatsappRest {

    Logger logger = Logger.getLogger(WhatsappRest.class.getName());
    Whatsapp whatsapp;
    WhatsappService whatsappService;
    WhatsappUtils utils;
    public WhatsappRest(WhatsappBean whatsapp, WhatsappService whatsappService) {
        this.whatsapp = whatsapp.getWhatsapp();
        this.whatsappService = whatsappService;
    }

    @GetMapping("qr")
    public ResponseEntity<?>  getQR() {
        if (whatsapp == null) {
            return ResponseEntity.status(Response.SC_OK).body(utils.generateQR());
        }
        whatsapp.connect().join();
        if (whatsapp.isConnected()) {
            return ResponseEntity.status(Response.SC_OK).body("Already connected");
        }else{
            return ResponseEntity.status(Response.SC_OK).body(utils.generateQR());
        }
    }

    @GetMapping("logout")
    public ResponseEntity<?>  logout() {
        if (whatsapp == null) {
            return ResponseEntity.status(Response.SC_OK).body("Not connected");
        }
        System.out.println(whatsapp.store().about());
        try {
            if(whatsapp.isConnected()){
                whatsapp.disconnect().join();
                whatsapp.logout().join();
                return ResponseEntity.status(Response.SC_OK).body("Logged out");
            }else{
                return ResponseEntity.status(Response.SC_OK).body("Not connected");
            }

        } catch (Exception e) {
            logger.severe(e.getMessage());
            return ResponseEntity.status(Response.SC_INTERNAL_SERVER_ERROR).body("Logout failed");
        }
    }

    @PostMapping(value = "send", consumes = MediaType.MULTIPART_FORM_DATA_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<?> send(@RequestParam Map<String, String> payload, @RequestParam Map<String, MultipartFile> files) {
        if (whatsapp == null) {
            return ResponseEntity.status(Response.SC_OK).body("Not connected");
        }
        try{
            whatsapp.connect().join();
            if (whatsapp.isConnected()) {
                whatsappService.send(payload, files, whatsapp);
                return ResponseEntity.status(Response.SC_OK).body(payload);
            }else{
                return ResponseEntity.status(Response.SC_OK).body("Not connected");
            }
        }catch (Exception e){
            logger.severe(e.getMessage());
            return ResponseEntity.status(Response.SC_INTERNAL_SERVER_ERROR).body("Send failed");
        }
    }

}
